@@ -33,8 +33,7 @@
 #include <gdk/gdkkeysyms.h>
 #include <X11/Xlib.h>
 #include <gdk/gdkx.h>
-#include <libgnomevfs/gnome-vfs.h>
-#include <libgnomevfs/gnome-vfs-mime-handlers.h>
+#include <gio/gio.h>
 
 #include "tile.h"
 #include "application-tile.h"
@@ -95,6 +94,7 @@ typedef struct {
 
 	GtkWidget *search_section;
 	GtkWidget *search_entry;
+	GtkWidget *network_status;
 
 	GtkNotebook *file_section;
 	GtkWidget   *page_selectors    [3];
@@ -116,9 +116,12 @@ typedef struct {
 
 	BookmarkAgent *bm_agents [BOOKMARK_STORE_N_TYPES];
 
-	GnomeVFSVolumeMonitor *volume_mon;
+	GVolumeMonitor        *volume_mon;
 	GList                 *mounts;
 
+	GFileMonitor *recently_used_store_monitor;
+	guint recently_used_timeout_id;
+
 	guint search_cmd_gconf_mntr_id;
 	guint current_page_gconf_mntr_id;
 	guint more_link_vis_gconf_mntr_id;
@@ -136,6 +139,8 @@ typedef struct {
 
 	gboolean ptr_is_grabbed;
 	gboolean kbd_is_grabbed;
+
+	guint recently_used_store_has_changed : 1;
 } MainMenuUIPrivate;
 
 #define PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), MAIN_MENU_UI_TYPE, MainMenuUIPrivate))
@@ -157,6 +162,8 @@ static void create_more_buttons      (MainMenuUI *);
 static void setup_file_tables        (MainMenuUI *);
 static void setup_bookmark_agents    (MainMenuUI *);
 static void setup_lock_down          (MainMenuUI *);
+static void setup_recently_used_store_monitor (MainMenuUI *this, gboolean is_startup);
+static void update_recently_used_sections (MainMenuUI *this);
 
 static void       select_page                (MainMenuUI *);
 static void       update_limits              (MainMenuUI *);
@@ -211,7 +218,7 @@ static void     search_tomboy_bindkey_cb          (gchar *, gpointer);
 static gboolean grabbing_window_event_cb          (GtkWidget *, GdkEvent *, gpointer);
 static void     user_app_agent_notify_cb          (GObject *, GParamSpec *, gpointer);
 static void     user_doc_agent_notify_cb          (GObject *, GParamSpec *, gpointer);
-static void     volume_monitor_mount_cb           (GnomeVFSVolumeMonitor *, GnomeVFSVolume *, gpointer);
+static void     volume_monitor_mount_cb           (GVolumeMonitor *, GMount *, gpointer);
 
 static GdkFilterReturn slab_gdk_message_filter (GdkXEvent *, GdkEvent *, gpointer);
 
@@ -277,25 +284,44 @@ main_menu_ui_new (PanelApplet *applet)
 	priv->panel_button_xml = glade_xml_new (glade_xml_path, "slab-panel-button-root", NULL);
 	g_free (glade_xml_path);
 
+	libslab_checkpoint ("main_menu_ui_new(): setup_recently_used_store_monitor");
+	setup_recently_used_store_monitor (this, TRUE);
+	libslab_checkpoint ("main_menu_ui_new(): setup_bookmark_agents");
 	setup_bookmark_agents    (this);
+	libslab_checkpoint ("main_menu_ui_new(): create_panel_button");
 	create_panel_button      (this);
+	libslab_checkpoint ("main_menu_ui_new(): create_slab_window");
 	create_slab_window       (this);
+	libslab_checkpoint ("main_menu_ui_new(): create_search_section");
 	create_search_section    (this);
+	libslab_checkpoint ("main_menu_ui_new(): create_file_section");
 	create_file_section      (this);
+	libslab_checkpoint ("main_menu_ui_new(): create_user_apps_section");
 	create_user_apps_section (this);
+	libslab_checkpoint ("main_menu_ui_new(): create_rct_apps_section");
 	create_rct_apps_section  (this);
+	libslab_checkpoint ("main_menu_ui_new(): create_user_docs_section");
 	create_user_docs_section (this);
+	libslab_checkpoint ("main_menu_ui_new(): create_rct_docs_section");
 	create_rct_docs_section  (this);
+	libslab_checkpoint ("main_menu_ui_new(): create_user_dirs_section");
 	create_user_dirs_section (this);
+	libslab_checkpoint ("main_menu_ui_new(): create_system_section");
 	create_system_section    (this);
+	libslab_checkpoint ("main_menu_ui_new(): create_status_section");
 	create_status_section    (this);
+	libslab_checkpoint ("main_menu_ui_new(): create_more_buttons");
 	create_more_buttons      (this);
+	libslab_checkpoint ("main_menu_ui_new(): setup_file_tables");
 	setup_file_tables        (this);
+	libslab_checkpoint ("main_menu_ui_new(): setup_lock_down");
 	setup_lock_down          (this);
 
+	libslab_checkpoint ("main_menu_ui_new(): bind_beagle_search_key");
 	bind_beagle_search_key  (this);
-	update_limits           (this);
+	libslab_checkpoint ("main_menu_ui_new(): select_page");
 	select_page             (this);
+	libslab_checkpoint ("main_menu_ui_new(): apply_lockdown_settings");
 	apply_lockdown_settings (this);
 
 	return this;
@@ -373,6 +399,7 @@ main_menu_ui_init (MainMenuUI *this)
 
 	priv->status_section                             = NULL;
 	priv->system_section                             = NULL;
+	priv->network_status                             = NULL;
 
 	priv->volume_mon                                 = NULL;
 
@@ -404,6 +431,8 @@ main_menu_ui_finalize (GObject *g_obj)
 
 	gint i;
 
+	if (priv->recently_used_store_monitor)
+		g_file_monitor_cancel (priv->recently_used_store_monitor);
 
 	for (i = 0; i < 4; ++i) {
 		g_object_unref (G_OBJECT (g_object_get_data (
@@ -437,9 +466,9 @@ main_menu_ui_finalize (GObject *g_obj)
 	for (i = 0; i < BOOKMARK_STORE_N_TYPES; ++i)
 		g_object_unref (priv->bm_agents [i]);
 
-	g_list_foreach (priv->mounts, (GFunc) gnome_vfs_volume_unref, NULL);
+	g_list_foreach (priv->mounts, (GFunc) g_object_unref, NULL);
 	g_list_free (priv->mounts);
-	gnome_vfs_volume_monitor_unref (priv->volume_mon);
+	g_object_unref (priv->volume_mon);
 
 	G_OBJECT_CLASS (main_menu_ui_parent_class)->finalize (g_obj);
 }
@@ -694,15 +723,15 @@ create_status_section (MainMenuUI *this)
 
 	ctnr = GTK_CONTAINER (glade_xml_get_widget (
 		priv->main_menu_xml, "network-status-container"));
-	tile = network_status_tile_new ();
+	priv->network_status = network_status_tile_new ();
 
-	gtk_widget_set_size_request (tile, 6 * icon_width, -1);
+	gtk_widget_set_size_request (priv->network_status, 6 * icon_width, -1);
 
 	g_signal_connect (
-		G_OBJECT (tile), "tile-action-triggered",
+		G_OBJECT (priv->network_status), "tile-action-triggered",
 		G_CALLBACK (tile_action_triggered_cb), this);
 
-	gtk_container_add   (ctnr, tile);
+	gtk_container_add   (ctnr, priv->network_status);
 	gtk_widget_show_all (GTK_WIDGET (ctnr));
 
 	priv->status_section = glade_xml_get_widget (
@@ -780,11 +809,11 @@ create_rct_docs_section (MainMenuUI *this)
 
 	gtk_container_add (ctnr, GTK_WIDGET (priv->file_tables [RCNT_DOCS_TABLE]));
 
-	priv->volume_mon = gnome_vfs_get_volume_monitor ();
-	priv->mounts = gnome_vfs_volume_monitor_get_mounted_volumes (priv->volume_mon);
+	priv->volume_mon = g_volume_monitor_get ();
+	priv->mounts = g_volume_monitor_get_mounts (priv->volume_mon);
 
-	g_signal_connect (priv->volume_mon, "volume_mounted", G_CALLBACK (volume_monitor_mount_cb), this);
-	g_signal_connect (priv->volume_mon, "volume_unmounted", G_CALLBACK (volume_monitor_mount_cb), this);
+	g_signal_connect (priv->volume_mon, "mount-added", G_CALLBACK (volume_monitor_mount_cb), this);
+	g_signal_connect (priv->volume_mon, "mount-removed", G_CALLBACK (volume_monitor_mount_cb), this);
 }
 
 static void
@@ -921,6 +950,79 @@ setup_lock_down (MainMenuUI *this)
 		DISABLE_LOCKSCREEN_GCONF_KEY, lockdown_notify_cb, this);
 }
 
+static char *
+get_recently_used_store_filename (void)
+{
+	const char *basename;
+
+	basename = ".recently-used.xbel";
+
+	return g_build_filename (g_get_home_dir (), basename, NULL);
+}
+
+#define RECENTLY_USED_STORE_THROTTLE_MILLISECONDS 2000
+
+static void recently_used_store_monitor_changed_cb (GFileMonitor *monitor,
+						    GFile *f1, GFile *f2,
+						    GFileMonitorEvent event_type,
+						    gpointer data)
+{
+	MainMenuUI *this = MAIN_MENU_UI (data);
+	MainMenuUIPrivate *priv = PRIVATE (this);
+
+	priv->recently_used_store_has_changed = TRUE;
+	update_recently_used_sections (this);
+}
+
+static gboolean
+setup_recently_used_throttle_timeout (gpointer data)
+{
+	MainMenuUI *this = data;
+	MainMenuUIPrivate *priv = PRIVATE (this);
+
+	update_recently_used_sections (this);
+	priv->recently_used_timeout_id = 0;
+
+	return FALSE;
+}
+
+/* Creates a GFileMonitor for the recently-used store, so we can be informed
+ * when the store changes.
+ */
+static void
+setup_recently_used_store_monitor (MainMenuUI *this, gboolean is_startup)
+{
+	MainMenuUIPrivate *priv = PRIVATE (this);
+	char *path;
+	GFile *file;
+	GFileMonitor *monitor;
+
+	priv->recently_used_store_has_changed = TRUE; /* ensure the store gets read the first time we need it */
+
+	path = get_recently_used_store_filename ();
+	file = g_file_new_for_path (path);
+
+	monitor = g_file_monitor_file (file, 0, NULL, NULL);
+	if (monitor) {
+		g_file_monitor_set_rate_limit (monitor,
+					       RECENTLY_USED_STORE_THROTTLE_MILLISECONDS);
+		g_signal_connect (monitor, "changed",
+				  G_CALLBACK (recently_used_store_monitor_changed_cb),
+				  this);
+	}
+
+	priv->recently_used_store_monitor = monitor;
+
+	if (priv->recently_used_timeout_id != 0)
+		g_source_remove (priv->recently_used_timeout_id);
+
+	if (is_startup)
+		priv->recently_used_timeout_id = g_idle_add (setup_recently_used_throttle_timeout, this);
+	else
+		priv->recently_used_timeout_id = g_timeout_add_seconds (RECENTLY_USED_STORE_THROTTLE_MILLISECONDS / 1000, /* 2 seconds */
+									setup_recently_used_throttle_timeout, this);
+}
+
 static Tile *
 item_to_user_app_tile (BookmarkItem *item, gpointer data)
 {
@@ -954,6 +1056,16 @@ item_to_recent_app_tile (BookmarkItem *item, gpointer data)
 static Tile *
 item_to_user_doc_tile (BookmarkItem *item, gpointer data)
 {
+	//force an icon since the thumbnail is basically blank - BNC#373783
+	if (item->title && (!strcmp (item->title, "BLANK_SPREADSHEET") || !strcmp (item->title, "BLANK_DOCUMENT")))
+	{
+		if (! strcmp (item->title, "BLANK_SPREADSHEET"))
+			return TILE (document_tile_new_force_icon (item->uri, item->mime_type,
+				item->mtime, "gnome-mime-application-vnd.oasis.opendocument.spreadsheet-template"));
+		else
+			return TILE (document_tile_new_force_icon (item->uri, item->mime_type,
+				item->mtime, "gnome-mime-application-vnd.oasis.opendocument.text-template"));
+	}
 	return TILE (document_tile_new (item->uri, item->mime_type, item->mtime));
 }
 
@@ -962,31 +1074,35 @@ item_to_recent_doc_tile (BookmarkItem *item, gpointer data)
 {
 	MainMenuUIPrivate *priv = PRIVATE (data);
 
-	GnomeVFSVolume *vol;
+	GMount         *mount;
+	GVolume        *volume;
+	GFile          *file;
+	char           *nfs_id;
 	gboolean        is_nfs = FALSE;
-	GnomeVFSURI    *gvfs_uri;
 	gboolean        is_local = TRUE;
 
 	GList *node;
 
-
 	if (! g_str_has_prefix (item->uri, "file://"))
 		return NULL;
 
 	for (node = priv->mounts; ! is_nfs && node; node = node->next) {
-		vol = (GnomeVFSVolume *) node->data;
+		mount = node->data;
+		volume = g_mount_get_volume (mount);
 
-		is_nfs =
-			((gnome_vfs_volume_get_device_type (vol) == GNOME_VFS_DEVICE_TYPE_NFS) &&
-			g_str_has_prefix (item->uri, gnome_vfs_volume_get_activation_uri (vol)));
+		nfs_id = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_NFS_MOUNT);
+		is_nfs = (nfs_id != NULL);
+
+		g_free (nfs_id);
+		g_object_unref (volume);
 	}
 
 	if (is_nfs)
 		return NULL;
 
-	gvfs_uri = gnome_vfs_uri_new (item->uri);
-	is_local = gnome_vfs_uri_is_local (gvfs_uri);
-	gnome_vfs_uri_unref (gvfs_uri);
+	file = g_file_new_for_uri (item->uri);
+	is_local = g_file_is_native (file);
+	g_object_unref (file);
 
 	if (! is_local)
 		return NULL;
@@ -1000,7 +1116,7 @@ item_to_recent_doc_tile (BookmarkItem *item, gpointer data)
 static Tile *
 item_to_dir_tile (BookmarkItem *item, gpointer data)
 {
-	return TILE (directory_tile_new (item->uri, item->title, item->icon));
+	return TILE (directory_tile_new (item->uri, item->title, item->icon, item->mime_type));
 }
 
 static Tile *
@@ -1047,34 +1163,39 @@ static BookmarkItem *
 doc_uri_to_item (const gchar *uri, gpointer data)
 {
 	BookmarkItem *item;
-
-	GnomeVFSFileInfo        *info;
-	GnomeVFSMimeApplication *default_app;
-
+	GFile *file;
+	GFileInfo *info;
+	GAppInfo *default_app;
 
 	item = g_new0 (BookmarkItem, 1);
 
 	item->uri = g_strdup (uri);
 
-	info = gnome_vfs_file_info_new ();
+	file = g_file_new_for_uri (uri);
+	info = g_file_query_info (file,
+				  G_FILE_ATTRIBUTE_TIME_MODIFIED ","
+				  G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+				  0, NULL, NULL);
+	g_object_unref (file);
 
-	gnome_vfs_get_file_info (
-		item->uri, info, 
-		GNOME_VFS_FILE_INFO_GET_MIME_TYPE | GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE);
-
-	item->mime_type = g_strdup (info->mime_type);
-	item->mtime     = info->mtime;
-
-	if (item->mime_type) {
-		default_app = gnome_vfs_mime_get_default_application (item->mime_type);
-
-		item->app_name = g_strdup (gnome_vfs_mime_application_get_name (default_app));
-		item->app_exec = g_strdup (gnome_vfs_mime_application_get_exec (default_app));
-
-		gnome_vfs_mime_application_free (default_app);
+	if (!info) {
+		item->mtime = 0;
+		item->mime_type = NULL;
+	} else {
+		item->mtime = (time_t) g_file_info_get_attribute_uint64 (info,
+									 G_FILE_ATTRIBUTE_TIME_MODIFIED);
+		item->mime_type = g_strdup (g_file_info_get_content_type (info));
+		g_object_unref (info);
 	}
 
-	gnome_vfs_file_info_unref (info);
+	if (item->mime_type) {
+		default_app = g_app_info_get_default_for_type (item->mime_type, FALSE);
+
+		item->app_name = g_strdup (g_app_info_get_name (default_app));
+		item->app_exec = g_strdup (g_app_info_get_executable (default_app));
+
+		g_object_unref (default_app);
+	}
 
 	if (! (item->mime_type && item->app_name)) {
 		bookmark_item_free (item);
@@ -1492,6 +1613,7 @@ apply_lockdown_settings (MainMenuUI *this)
 	GList *node;
 	gint   i;
 
+	libslab_checkpoint ("apply_lockdown_settings(): start");
 
 	more_link_visible   = GPOINTER_TO_INT (libslab_get_gconf_value (MORE_LINK_VIS_GCONF_KEY));
 	status_area_visible = GPOINTER_TO_INT (libslab_get_gconf_value (STATUS_VIS_GCONF_KEY));
@@ -1531,13 +1653,22 @@ apply_lockdown_settings (MainMenuUI *this)
 	for (i = 0; i < 5; ++i)
 		set_table_section_visible (this, priv->file_tables [i]);
 
+	libslab_checkpoint ("apply_lockdown_settings(): loading sys_table");
 	tile_table_reload (priv->sys_table);
+
+	libslab_checkpoint ("apply_lockdown_settings(): loading user_apps_table");
 	tile_table_reload (priv->file_tables [USER_APPS_TABLE]);
-	tile_table_reload (priv->file_tables [RCNT_APPS_TABLE]);
+
+	libslab_checkpoint ("apply_lockdown_settings(): loading user_docs_table");
 	tile_table_reload (priv->file_tables [USER_DOCS_TABLE]);
+
+	libslab_checkpoint ("apply_lockdown_settings(): loading user_dirs_table");
 	tile_table_reload (priv->file_tables [USER_DIRS_TABLE]);
 
+	libslab_checkpoint ("apply_lockdown_settings(): update_limits");
 	update_limits (this);
+
+	libslab_checkpoint ("apply_lockdown_settings(): end");
 }
 
 static void
@@ -1616,6 +1747,95 @@ exit:
 	xmlFreeDoc (doc);
 }
 
+static GBookmarkFile *
+load_recently_used_store (void)
+{
+	GBookmarkFile *store;
+	char *filename;
+
+	store = g_bookmark_file_new ();
+
+	filename = get_recently_used_store_filename ();
+
+	/* FIXME: if we can't load the store, do we need to hide the
+	 * recently-used sections in the GUI?  If so, do that in the caller(s)
+	 * of this function, not here.
+	 */
+
+	libslab_checkpoint ("main-menu-ui.c: load_recently_used_store(): start loading %s", filename);
+	g_bookmark_file_load_from_file (store, filename, NULL); /* NULL-GError */
+	libslab_checkpoint ("main-menu-ui.c: load_recently_used_store(): end loading %s", filename);
+
+	g_free (filename);
+
+	return store;
+}
+
+/* Updates the bookmark agents for the recently-used apps and documents, by reading the
+ * recently-used store.
+ */
+static void
+update_recently_used_bookmark_agents (MainMenuUI *this)
+{
+	MainMenuUIPrivate *priv = PRIVATE (this);
+	GBookmarkFile *store;
+
+	store = load_recently_used_store ();
+
+	bookmark_agent_update_from_bookmark_file (priv->bm_agents[BOOKMARK_STORE_RECENT_APPS], store);
+	bookmark_agent_update_from_bookmark_file (priv->bm_agents[BOOKMARK_STORE_RECENT_DOCS], store);
+
+	g_bookmark_file_free (store);
+}
+
+/* Updates the recently-used tile tables from their corresponding bookmark agents */
+static void
+update_recently_used_tables (MainMenuUI *this)
+{
+	MainMenuUIPrivate *priv = PRIVATE (this);
+
+	tile_table_reload (priv->file_tables[RCNT_APPS_TABLE]);
+	tile_table_reload (priv->file_tables[RCNT_DOCS_TABLE]);
+}
+
+/* If the recently-used store has changed since the last time we updated from
+ * it, this updates our view of the store and the corresponding sections in the
+ * slab_window.
+ */
+static void
+update_recently_used_sections (MainMenuUI *this)
+{
+	MainMenuUIPrivate *priv = PRIVATE (this);
+
+	libslab_checkpoint ("main-menu-ui.c: update_recently_used_sections() start");
+
+	if (priv->recently_used_store_has_changed) {
+		update_recently_used_bookmark_agents (this);
+		update_recently_used_tables (this);
+
+		priv->recently_used_store_has_changed = FALSE;
+	}
+
+	if (!priv->recently_used_store_monitor)
+		setup_recently_used_store_monitor (this, FALSE);
+
+	libslab_checkpoint ("main-menu-ui.c: update_recently_used_sections() end");
+}
+
+/* Updates the slab_window's sections that need updating and presents the window */
+static void
+present_slab_window (MainMenuUI *this)
+{
+	MainMenuUIPrivate *priv = PRIVATE (this);
+
+	network_tile_update_status (priv->network_status);
+
+	update_recently_used_sections (this);
+
+	gtk_window_set_screen (GTK_WINDOW (priv->slab_window), gtk_widget_get_screen (GTK_WIDGET (priv->panel_applet)));
+	gtk_window_present_with_time (GTK_WINDOW (priv->slab_window), gtk_get_current_event_time ());
+}
+
 static void
 panel_button_clicked_cb (GtkButton *button, gpointer user_data)
 {
@@ -1636,7 +1856,7 @@ panel_button_clicked_cb (GtkButton *button, gpointer user_data)
 
 	if (! double_click_detector_is_double_click (detector, gtk_get_current_event_time (), TRUE)) {
 		if (! visible)
-			gtk_window_present_with_time (GTK_WINDOW (priv->slab_window), gtk_get_current_event_time ());
+			present_slab_window (this);
 		else
 			gtk_widget_hide (priv->slab_window);
 
@@ -2282,11 +2502,11 @@ user_doc_agent_notify_cb (GObject *g_obj, GParamSpec *pspec, gpointer user_data)
 }
 
 static void
-volume_monitor_mount_cb (GnomeVFSVolumeMonitor *mon, GnomeVFSVolume *vol, gpointer data)
+volume_monitor_mount_cb (GVolumeMonitor *mon, GMount *mount, gpointer data)
 {
 	MainMenuUIPrivate *priv = PRIVATE (data);
 
-	g_list_foreach (priv->mounts, (GFunc) gnome_vfs_volume_unref, NULL);
+	g_list_foreach (priv->mounts, (GFunc) g_object_unref, NULL);
 	g_list_free (priv->mounts);
-	priv->mounts = gnome_vfs_volume_monitor_get_mounted_volumes (mon);
+	priv->mounts = g_volume_monitor_get_mounts (mon);
 }
